----------------------------------------------------------------------
--  Adadep_Options package body                                     --
--  Copyright (C) 2002, 2016 Adalog                                 --
--  Author: J-P. Rosen                                              --
--                                                                  --
--  ADALOG   is   providing   training,   consultancy,   expertise, --
--  assistance and custom developments  in Ada and related software --
--  engineering techniques.  For more info about our services:      --
--  ADALOG                          Tel: +33 1 45 29 21 52          --
--  2 rue du Docteur Lombard        Fax: +33 1 45 29 25 00          --
--  92441 ISSY LES MOULINEAUX CEDEX E-m: info@adalog.fr             --
--  FRANCE                          URL: http://www.adalog.fr       --
--                                                                  --
--  This  unit is  free software;  you can  redistribute  it and/or --
--  modify  it under  terms of  the GNU  General Public  License as --
--  published by the Free Software Foundation; either version 2, or --
--  (at your  option) any later version.  This  unit is distributed --
--  in the hope  that it will be useful,  but WITHOUT ANY WARRANTY; --
--  without even the implied warranty of MERCHANTABILITY or FITNESS --
--  FOR A  PARTICULAR PURPOSE.  See the GNU  General Public License --
--  for more details.   You should have received a  copy of the GNU --
--  General Public License distributed  with this program; see file --
--  COPYING.   If not, write  to the  Free Software  Foundation, 59 --
--  Temple Place - Suite 330, Boston, MA 02111-1307, USA.           --
--                                                                  --
--  As  a special  exception, if  other files  instantiate generics --
--  from  this unit,  or you  link this  unit with  other  files to --
--  produce an executable,  this unit does not by  itself cause the --
--  resulting executable  to be covered  by the GNU  General Public --
--  License.  This exception does  not however invalidate any other --
--  reasons why  the executable  file might be  covered by  the GNU --
--  Public License.                                                 --
----------------------------------------------------------------------

with   -- Standard Ada units
  Ada.Characters.Handling,
  Ada.Exceptions;

with   -- ASIS
  Asis.Implementation;

with   -- Other reusable components
  Options_Analyzer,
  Utilities;

with   -- Application specific units
  Implementation_Options,
  Analyzer;

package body Adadep_Options is

   package Options is new Options_Analyzer (Binary_Options => "bdhmrsvw",
                                            Valued_Options => "op",
                                            Tail_Separator => "--");
   use Options;

   ----------------
   -- Print_Help --
   ----------------

   procedure Print_Help is
      use Asis.Implementation, Utilities;
   begin
      User_Message ("Adadep V" & Version & " with " & ASIS_Implementor_Version & ", (c) Adalog 2002, 2016");
      User_Message ("For information about Adalog's services, please see http://www.adalog.fr");
      User_Message ("Usage: adadep [-bdmrsw] [-o <out-file>] [-p <project-file>] " &
                    " <unit>{+|-<unit>}|@<file> {<element>}" &
                    " [-- <ASIS options>]");
      User_Message ("   or: adadep -h");
   end Print_Help;


   -------------------
   -- Output_Option --
   -------------------

   function Output_Option return String is
   begin
      return Value (Option => 'o');
   end Output_Option;

   ---------------------
   -- Analyze_Options --
   ---------------------

   procedure Analyze_Options is
      use Ada.Exceptions, Ada.Characters.Handling;
      use Utilities, Analyzer;
   begin
      --
      -- Get rid of "help" case
      --
      if Is_Present (Option => 'h') then
         Print_Help;
         raise Cancellation;
      end if;


      -- Initialize options
      Utilities.Debug_Option   := Is_Present (Option => 'd');
      Utilities.Verbose_Option := Is_Present (Option => 'v');

      Body_Option      := Is_Present (Option => 'b');
      Spec_Option      := Is_Present (Option => 's');
      Merge_Option     := Is_Present (Option => 'm');
      Recursive_Option := Is_Present (Option => 'r');
      Overwrite_Option := Is_Present (Option => 'w');

      -- Check consistency
      if Parameter_Count = 0 then
         Raise_Exception (Options_Error'Identity, "Must provide unit name");
      end if;

      -- If no -s or -b option provided, handle both
      if not (Body_Option or Spec_Option) then
         Spec_Option := True;
         Body_Option := True;
      end if;

      -- Add list of kept elements if any
      for I in Natural range 2..Parameter_Count loop
         Add_Kept_Element (To_Wide_String (Parameter (I)));
      end loop;

   exception
      when Occur : Options_Error =>
         User_Message (To_Wide_String (Ada.Exceptions.Exception_Message (Occur)));
         Print_Help;
         raise Cancellation;
   end Analyze_Options;


   -----------------------
   -- Initialize_String --
   -----------------------

   function Initialize_String return Wide_String is
      use Implementation_Options, Utilities;
   begin
      return Initialize_String (Debug_Option);
   end Initialize_String;


   -----------------------
   -- Parameters_String --
   -----------------------

   function Parameters_String return Wide_String is
      use Ada.Characters.Handling;
      use Implementation_Options;
   begin
      -- The following String' qualification is needed to what looks
      -- like a Gnat bug. Anyway, it's harmless.
      return Parameters_String (Project_File => String'(Value (Option => 'p',
                                                               Explicit_Required => True)),
                                Other_Options => To_Wide_String (Options.Tail_Value));
   exception
      when Occur : Implementation_Error | Options_Error =>
         -- -p file not found or incorrect
         Utilities.User_Message ("Option error: " & To_Wide_String (Ada.Exceptions.Exception_Message (Occur)));
         Utilities.User_Message ("try -h for help");
         raise Cancellation;
   end Parameters_String;


   ------------------
   -- Units_String --
   ------------------

   function Units_String return Wide_String is
      use Ada.Characters.Handling;
   begin
      return To_Wide_String (Parameter (1));
   end Units_String;


end Adadep_Options;
